import os

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class DBHistoryMigration(MigrationABC):
    name = "1.0.0_DBHistoryMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/api_keys.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auth_users.sql").read()
        )
        self._cursor.execute(
            open(
                f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auth_user_users_relation.sql"
            ).read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auto_role_rules.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/auto_roles.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/clients.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/game_servers.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/known_users.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/levels.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/servers.sql").read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_game_idents.sql").read()
        )
        self._cursor.execute(
            open(
                f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_joined_game_servers.sql"
            ).read()
        )
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_joined_servers.sql").read()
        )
        self._cursor.execute(
            open(
                f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_joined_voice_channel.sql"
            ).read()
        )
        self._cursor.execute(
            open(
                f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_message_count_per_hour.sql"
            ).read()
        )
        self._cursor.execute(open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/users.sql").read())
        self._cursor.execute(
            open(f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts/user_warnings.sql").read()
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ApiKeysHistory`;")
        self._cursor.execute("DROP TABLE `AuthUsersHistory`;")
        self._cursor.execute("DROP TABLE `AuthUserUsersRelationsHistory`;")
        self._cursor.execute("DROP TABLE `AutoRoleRulesHistory`;")
        self._cursor.execute("DROP TABLE `AutoRolesHistory`;")
        self._cursor.execute("DROP TABLE `ClientsHistory`;")
        self._cursor.execute("DROP TABLE `GameServersHistory`;")
        self._cursor.execute("DROP TABLE `KnownUsersHistory`;")
        self._cursor.execute("DROP TABLE `LevelsHistory`;")
        self._cursor.execute("DROP TABLE `ServersHistory`;")
        self._cursor.execute("DROP TABLE `UserGameIdentsHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedGameServerHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedServersHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedVoiceChannelHistory`;")
        self._cursor.execute("DROP TABLE `UserMessageCountPerHourHistory`;")
        self._cursor.execute("DROP TABLE `UsersHistory`;")
        self._cursor.execute("DROP TABLE `UserWarningsHistory`;")
