from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class UserJoinedGameServerMigration(MigrationABC):
    name = "1.0_UserJoinedGameServerMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                CREATE TABLE IF NOT EXISTS `GameServers` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `Name` VARCHAR(255) NOT NULL,
                `ServerId` BIGINT NOT NULL,
                `ApiKeyId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`ServerId`) REFERENCES Servers(`ServerId`),
                FOREIGN KEY (`ApiKeyId`) REFERENCES ApiKeys(`Id`),
                PRIMARY KEY(`Id`)
            );
            """
            )
        )

        self._cursor.execute(
            str(
                f"""
                CREATE TABLE IF NOT EXISTS `UserJoinedGameServer` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `GameServerId` BIGINT NOT NULL,
                `JoinedOn` DATETIME(6) NOT NULL,
                `LeavedOn` DATETIME(6),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                FOREIGN KEY (`GameServerId`) REFERENCES GameServers(`Id`),
                PRIMARY KEY(`Id`)
            );
            """
            )
        )

        self._cursor.execute(
            str(
                f"""
                CREATE TABLE IF NOT EXISTS `UserGameIdents` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `GameServerId` BIGINT NOT NULL,
                `Ident` VARCHAR(255) NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                FOREIGN KEY (`GameServerId`) REFERENCES GameServers(`Id`),
                CONSTRAINT UC_UserGameIdent UNIQUE (`GameServerId`,`Ident`),
                PRIMARY KEY(`Id`)
            );
            """
            )
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `GameServers`;")
        self._cursor.execute("DROP TABLE `UserJoinedGameServer`;")
        self._cursor.execute("DROP TABLE `UserGameIdents`;")
