from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_graphql.abc.data_query_abc import DataQueryABC
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.server_filter import ServerFilter


class AutoRoleQuery(DataQueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        auto_role_rules: AutoRoleRepositoryABC,
        servers: ServerRepositoryABC,
    ):
        DataQueryABC.__init__(self, "AutoRole")

        self._bot = bot
        self._auto_role_rules = auto_role_rules
        self._servers = servers

        self.set_field("id", self.resolve_id)
        self.set_field("channelId", self.resolve_channel_id)
        self.set_field("channelName", self.resolve_channel_name)
        self.set_field("messageId", self.resolve_message_id)
        self.set_field("server", self.resolve_server)
        self.add_collection(
            "autoRoleRule",
            lambda x, *_: self._auto_role_rules.get_auto_role_rules_by_auto_role_id(x.id),
            AutoRoleFilter,
        )

    @staticmethod
    def resolve_id(x: AutoRole, *_):
        return x.id

    @staticmethod
    def resolve_channel_id(x: AutoRole, *_):
        return x.discord_channel_id

    @staticmethod
    def resolve_channel_name(x: AutoRole, *_):
        return x.discord_channel_name

    @staticmethod
    def resolve_message_id(x: AutoRole, *_):
        return x.discord_message_id

    def resolve_server(self, x: AutoRole, *_, filter: ServerFilter = None):
        if filter is not None:
            return filter.filter(self._servers.get_server_by_id(x.server.id))

        return self._servers.get_server_by_id(x.server.id)
